'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.clearCachedDOM = clearCachedDOM;

var _jsdom = require('jsdom');

var _Logger = require('./Logger');

var _Logger2 = _interopRequireDefault(_Logger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const { VERBOSE } = process.env;

const MAX_ERROR_DETAIL_LENGTH = 200;

// Cache the JSDOM instance because re-loading the webpack bundle for every
// target can be very expensive. This assumes that jsdomOptions and
// webpackBundle do not change.
let dom;
function getCachedDOM(jsdomOptions, webpackBundle) {
  if (!dom) {
    const logger = new _Logger2.default();
    logger.start('Initializing JSDOM with the bundle...');

    const virtualConsole = new _jsdom.VirtualConsole();
    virtualConsole.on('jsdomError', e => {
      const { stack, detail = '' } = e;
      if (VERBOSE || typeof detail !== 'string' || detail.length < MAX_ERROR_DETAIL_LENGTH) {
        console.error(stack, detail);
      } else {
        const newDetail = `${(detail || '').slice(0, MAX_ERROR_DETAIL_LENGTH)}...
          To see the full error, run happo with "VERBOSE=true")`;
        console.error(stack, newDetail);
      }
    });
    virtualConsole.sendTo(console, { omitJSDOMErrors: true });

    dom = new _jsdom.JSDOM(`
        <!DOCTYPE html>
        <html>
          <head>
            <script src='file://${webpackBundle}'></script>
          </head>
          <body>
          </body>
        </html>
      `.trim(), Object.assign({
      runScripts: 'dangerously',
      resources: 'usable',
      url: 'http://localhost',
      virtualConsole,
      beforeParse(win) {
        win.requestAnimationFrame = callback => setTimeout(callback, 0);
        win.cancelAnimationFrame = clearTimeout;
      }
    }, jsdomOptions));

    logger.success();
  }

  return dom;
}

// Useful for tests
function clearCachedDOM() {
  dom = undefined;
}

class JSDOMDomProvider {
  constructor(jsdomOptions, { webpackBundle }) {
    this.dom = getCachedDOM(jsdomOptions, webpackBundle);
  }

  async init({ targetName }) {
    if (!this.dom.window.happoProcessor) {
      await new Promise(resolve => {
        this.dom.window.onBundleReady = resolve;
      });
    }
    return this.dom.window.happoProcessor.init({ targetName });
  }

  resize({ width, height }) {
    this.dom.window.outerWidth = this.dom.window.innerWidth = width;
    this.dom.window.outerHeight = this.dom.window.innerHeight = height;
    Object.defineProperties(this.dom.window.screen, {
      width: { value: width, configurable: true },
      availWidth: { value: width, configurable: true },
      height: { value: height, configurable: true },
      availHeight: { value: height, configurable: true }
    });
    return this.dom.window.happoProcessor.reset();
  }

  next() {
    return this.dom.window.happoProcessor.next();
  }

  processCurrent() {
    return this.dom.window.happoProcessor.processCurrent();
  }

  extractCSS() {
    return this.dom.window.happoProcessor.extractCSS();
  }

  close() {
    // no-op
  }
}
exports.default = JSDOMDomProvider;