'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createHash = require('./createHash');

var _createHash2 = _interopRequireDefault(_createHash);

var _makeRequest = require('./makeRequest');

var _makeRequest2 = _interopRequireDefault(_makeRequest);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

const POLL_INTERVAL = 5000; // 5 secs
const VIEWPORT_PATTERN = /^([0-9]+)x([0-9]+)$/;

async function waitFor({ requestId, endpoint, apiKey, apiSecret }) {
  const { status, result } = await (0, _makeRequest2.default)({
    url: `${endpoint}/api/snap-requests/${requestId}`,
    method: 'GET',
    json: true
  }, { apiKey, apiSecret, maxTries: 5 });
  if (status === 'done') {
    return result.map(i => Object.assign({}, i, { snapRequestId: requestId }));
  }
  await new Promise(r => setTimeout(r, POLL_INTERVAL));
  return waitFor({ requestId, endpoint, apiKey, apiSecret });
}

const MIN_INTERNET_EXPLORER_WIDTH = 400;

class RemoteBrowserTarget {
  constructor(browserName, _ref) {
    let { viewport, chunks = 1, maxHeight } = _ref,
        otherOptions = _objectWithoutProperties(_ref, ['viewport', 'chunks', 'maxHeight']);

    const viewportMatch = viewport.match(VIEWPORT_PATTERN);
    if (!viewportMatch) {
      throw new Error(`Invalid viewport "${viewport}". Here's an example of a valid one: "1024x768".`);
    }

    const [_, width] = viewportMatch; // eslint-disable-line no-unused-vars

    if (browserName === 'edge' && width < MIN_INTERNET_EXPLORER_WIDTH) {
      throw new Error(`Invalid viewport width for the "edge" target (you provided ${width}). Smallest width it can handle is ${MIN_INTERNET_EXPLORER_WIDTH}.`);
    }

    this.chunks = chunks;
    this.browserName = browserName;
    this.viewport = viewport;
    this.maxHeight = maxHeight;
    this.otherOptions = otherOptions;
  }

  async execute({
    globalCSS,
    assetsPackage,
    staticPackage,
    snapPayloads,
    apiKey,
    apiSecret,
    endpoint,
    pages,
    asyncResults = false,
    targetName
  }) {
    const boundMakeRequest = async ({ slice, chunk, pageSlice }) => {
      const payloadString = JSON.stringify(_extends({
        viewport: this.viewport,
        maxHeight: this.maxHeight
      }, this.otherOptions, {
        globalCSS,
        snapPayloads: slice,
        chunk,
        staticPackage,
        assetsPackage,
        pages: pageSlice
      }));
      const payloadHash = (0, _createHash2.default)(payloadString + (pageSlice ? Math.random() : ''));
      return (0, _makeRequest2.default)({
        url: `${endpoint}/api/snap-requests`,
        method: 'POST',
        json: true,
        formData: {
          type: `browser-${this.browserName}`,
          targetName,
          payloadHash,
          payload: {
            options: {
              filename: 'payload.json',
              contentType: 'application/json'
            },
            value: payloadString
          }
        }
      }, { apiKey, apiSecret, maxTries: 5 });
    };
    const promises = [];
    const requestIds = [];
    if (staticPackage) {
      for (let i = 0; i < this.chunks; i += 1) {
        // We allow one `await` inside the loop here to avoid POSTing all payloads
        // to the server at the same time (thus reducing load a little).
        // eslint-disable-next-line no-await-in-loop
        const { requestId } = await boundMakeRequest({
          chunk: { index: i, total: this.chunks }
        });
        if (asyncResults) {
          requestIds.push(requestId);
        } else {
          promises.push(waitFor({ requestId, endpoint, apiKey, apiSecret }));
        }
      }
    } else if (pages) {
      const pagesPerChunk = Math.ceil(pages.length / this.chunks);
      for (let i = 0; i < this.chunks; i += 1) {
        const pageSlice = pages.slice(i * pagesPerChunk, i * pagesPerChunk + pagesPerChunk);
        // We allow one `await` inside the loop here to avoid POSTing all payloads
        // to the server at the same time (thus reducing load a little).
        // eslint-disable-next-line no-await-in-loop
        const { requestId } = await boundMakeRequest({ pageSlice });
        if (asyncResults) {
          requestIds.push(requestId);
        } else {
          promises.push(waitFor({ requestId, endpoint, apiKey, apiSecret }));
        }
      }
    } else {
      const snapsPerChunk = Math.ceil(snapPayloads.length / this.chunks);
      for (let i = 0; i < this.chunks; i += 1) {
        const slice = snapPayloads.slice(i * snapsPerChunk, i * snapsPerChunk + snapsPerChunk);
        // We allow one `await` inside the loop here to avoid POSTing all payloads
        // to the server at the same time (thus reducing load a little).
        // eslint-disable-next-line no-await-in-loop
        const { requestId } = await boundMakeRequest({ slice });
        if (asyncResults) {
          requestIds.push(requestId);
        } else {
          promises.push(waitFor({ requestId, endpoint, apiKey, apiSecret }));
        }
      }
    }

    if (asyncResults) {
      return requestIds;
    }

    const result = [];
    (await Promise.all(promises)).forEach(list => {
      result.push(...list);
    });

    return result;
  }
}
exports.default = RemoteBrowserTarget;