'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _readline = require('readline');

var _readline2 = _interopRequireDefault(_readline);

var _perf_hooks = require('perf_hooks');

var _JSDOMDomProvider = require('./JSDOMDomProvider');

var _JSDOMDomProvider2 = _interopRequireDefault(_JSDOMDomProvider);

var _Logger = require('./Logger');

var _Logger2 = _interopRequireDefault(_Logger);

var _MultipleErrors = require('./MultipleErrors');

var _MultipleErrors2 = _interopRequireDefault(_MultipleErrors);

var _constructReport = require('./constructReport');

var _constructReport2 = _interopRequireDefault(_constructReport);

var _createStaticPackage = require('./createStaticPackage');

var _createStaticPackage2 = _interopRequireDefault(_createStaticPackage);

var _createDynamicEntryPoint = require('./createDynamicEntryPoint');

var _createDynamicEntryPoint2 = _interopRequireDefault(_createDynamicEntryPoint);

var _createWebpackBundle = require('./createWebpackBundle');

var _createWebpackBundle2 = _interopRequireDefault(_createWebpackBundle);

var _loadCSSFile = require('./loadCSSFile');

var _loadCSSFile2 = _interopRequireDefault(_loadCSSFile);

var _prepareAssetsPackage = require('./prepareAssetsPackage');

var _prepareAssetsPackage2 = _interopRequireDefault(_prepareAssetsPackage);

var _processSnapsInBundle = require('./processSnapsInBundle');

var _processSnapsInBundle2 = _interopRequireDefault(_processSnapsInBundle);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const { VERBOSE = 'false' } = process.env;

function logTargetResults({ name, globalCSS, snapPayloads }) {
  const cssPath = _path2.default.join(_os2.default.tmpdir(), `happo-verbose-${name}.css`);
  const snippetsPath = _path2.default.join(_os2.default.tmpdir(), `happo-snippets-${name}.json`);
  _fs2.default.writeFileSync(cssPath, JSON.stringify(globalCSS));
  _fs2.default.writeFileSync(snippetsPath, JSON.stringify(snapPayloads));
  console.log(`Recorded CSS for target "${name}" can be found in ${cssPath}`);
  console.log(`Recorded HTML snippets for target "${name}" can be found in ${snippetsPath}`);
}

function waitForAnyKey() {
  _readline2.default.emitKeypressEvents(process.stdin);
  process.stdin.setRawMode(true);
  process.stdin.resume();

  return new Promise(resolve => {
    process.stdin.once('keypress', (_, key) => {
      if (key.ctrl && key.name === 'c') {
        process.exit();
      } else {
        process.stdin.setRawMode(false);
        process.stdin.pause();
        resolve();
      }
    });
  });
}

function resolveDomProvider({ plugins, jsdomOptions }) {
  const pluginWithProvider = plugins.find(({ DomProvider }) => !!DomProvider);
  if (pluginWithProvider) {
    return pluginWithProvider.DomProvider;
  }
  return _JSDOMDomProvider2.default.bind(_JSDOMDomProvider2.default, jsdomOptions);
}

async function executeTargetWithPrerender({
  name,
  targets,
  css,
  snapPayloads,
  publicFolders,
  cssBlocks,
  apiKey,
  apiSecret,
  endpoint,
  isAsync
}) {
  if (!snapPayloads.length) {
    console.warn(`No examples found for target ${name}, skipping`);
    return [];
  }
  const errors = snapPayloads.filter(p => p.isError);
  if (errors.length === 1) {
    throw errors[0];
  }
  if (errors.length > 1) {
    throw new _MultipleErrors2.default(errors);
  }

  const globalCSS = cssBlocks.concat([{ css }]);

  if (VERBOSE === 'true') {
    logTargetResults({ name, globalCSS, snapPayloads });
  }

  const assetsPackage = await (0, _prepareAssetsPackage2.default)({
    globalCSS,
    snapPayloads,
    publicFolders
  });
  snapPayloads.forEach(item => {
    delete item.assetPaths;
  });

  const result = await targets[name].execute({
    asyncResults: isAsync,
    targetName: name,
    assetsPackage,
    globalCSS,
    snapPayloads,
    apiKey,
    apiSecret,
    endpoint
  });
  return result;
}

async function generateScreenshots({
  apiKey,
  apiSecret,
  stylesheets,
  endpoint,
  targets,
  publicFolders,
  jsdomOptions,
  plugins,
  prerender,
  tmpdir,
  isAsync
}, bundleFile, logger) {
  const cssBlocks = await Promise.all(stylesheets.map(async sheet => {
    const { source, id, conditional } = typeof sheet === 'string' ? { source: sheet } : sheet;
    const result = {
      source,
      css: await (0, _loadCSSFile2.default)(source)
    };
    if (id) result.id = id;
    if (conditional) result.conditional = conditional;
    return result;
  }));
  plugins.forEach(({ css }) => {
    if (css) {
      cssBlocks.push({ css });
    }
  });

  const targetNames = Object.keys(targets);
  const tl = targetNames.length;
  const DomProvider = resolveDomProvider({ plugins, jsdomOptions });
  logger.info(`Generating screenshots in ${tl} target${tl > 1 ? 's' : ''}...`);
  try {
    const staticPackage = prerender ? undefined : await (0, _createStaticPackage2.default)({
      tmpdir,
      publicFolders
    });

    let results;
    if (prerender) {
      const prerenderPromises = [];
      for (const name of targetNames) {
        // These tasks are CPU-bound, and we need to be careful about how much
        // memory we are using at one time, so we want to run them serially.
        // eslint-disable-next-line no-await-in-loop
        const { css, snapPayloads } = await (0, _processSnapsInBundle2.default)(bundleFile, {
          targetName: name,
          publicFolders,
          viewport: targets[name].viewport,
          DomProvider
        });
        prerenderPromises.push((async () => {
          const startTime = _perf_hooks.performance.now();
          const result = await executeTargetWithPrerender({
            name,
            css,
            snapPayloads,
            targets,
            publicFolders,
            viewport: targets[name].viewport,
            cssBlocks,
            apiKey,
            apiSecret,
            endpoint,
            logger,
            isAsync
          });
          logger.start(`  - ${name}`, { startTime });
          logger.success();
          return { name, result };
        })());
      }

      results = await Promise.all(prerenderPromises);
    } else {
      results = await Promise.all(targetNames.map(async name => {
        const startTime = _perf_hooks.performance.now();
        const result = await targets[name].execute({
          asyncResults: isAsync,
          targetName: name,
          staticPackage,
          globalCSS: cssBlocks,
          apiKey,
          apiSecret,
          endpoint
        });
        logger.start(`  - ${name}`, { startTime });
        logger.success();
        return { name, result };
      }));
    }
    if (isAsync) {
      return results;
    }
    return (0, _constructReport2.default)(results);
  } catch (e) {
    logger.fail();
    throw e;
  }
}

exports.default = async function domRunner({
  apiKey,
  apiSecret,
  setupScript,
  customizeWebpackConfig,
  stylesheets,
  include,
  endpoint,
  targets,
  publicFolders,
  rootElementSelector,
  renderWrapperModule,
  prerender,
  type,
  plugins,
  tmpdir,
  jsdomOptions,
  asyncTimeout
}, { only, isAsync, onReady }) {
  const boundGenerateScreenshots = generateScreenshots.bind(null, {
    apiKey,
    apiSecret,
    stylesheets,
    endpoint,
    targets,
    publicFolders,
    prerender,
    only,
    isAsync,
    jsdomOptions,
    plugins,
    tmpdir
  });
  const logger = new _Logger2.default();
  logger.start('Searching for happo test files...');
  let entryFile;
  try {
    const entryPointResult = await (0, _createDynamicEntryPoint2.default)({
      setupScript,
      include,
      only,
      type,
      plugins,
      tmpdir,
      rootElementSelector,
      renderWrapperModule,
      asyncTimeout
    });
    entryFile = entryPointResult.entryFile;
    logger.success(`${entryPointResult.numberOfFilesProcessed} found`);
  } catch (e) {
    logger.fail(e);
    throw e;
  }

  const debugIndexHtml = _fs2.default.readFileSync(_path2.default.resolve(__dirname, 'debug.html'));
  _fs2.default.writeFileSync(_path2.default.resolve(tmpdir, 'index.html'), debugIndexHtml);

  logger.start('Creating bundle...');

  if (onReady) {
    let currentBuildPromise;
    let currentLogger;
    let currentWaitPromise;
    // We're in dev/watch mode
    (0, _createWebpackBundle2.default)(entryFile, { type, customizeWebpackConfig, plugins, tmpdir }, {
      onBuildReady: async bundleFile => {
        if (currentBuildPromise) {
          currentBuildPromise.cancelled = true;
          currentLogger.mute();
          if (currentWaitPromise) {
            currentWaitPromise.cancelled = true;
          } else {
            logger.divider();
            logger.info('Changes detected. Press any key to continue.');
          }
          const waitPromise = waitForAnyKey();
          currentWaitPromise = waitPromise;
          await waitPromise;
          if (waitPromise.cancelled) {
            return;
          }
          logger.divider();
        } else {
          logger.success();
        }
        currentWaitPromise = undefined;
        const mutableLogger = new _Logger2.default();
        const buildPromise = boundGenerateScreenshots(bundleFile, mutableLogger);
        currentLogger = mutableLogger;
        currentBuildPromise = buildPromise;
        try {
          const report = await buildPromise;
          if (!buildPromise.cancelled) {
            onReady(report);
          }
        } catch (e) {
          logger.error(e);
        }
      }
    });
    return;
  }

  const bundleFile = await (0, _createWebpackBundle2.default)(entryFile, { type, customizeWebpackConfig, plugins, tmpdir }, {});
  logger.success();
  return boundGenerateScreenshots(bundleFile, logger);
};